package solutions.bitbadger.documents.common

/**
 * The SQL dialect to use when building queries
 */
enum class Dialect {
    /** PostgreSQL */
    POSTGRESQL,
    /** SQLite */
    SQLITE;

    companion object {

        /**
         * Derive the dialect from the given connection string
         *
         * @param connectionString The connection string from which the dialect will be derived
         * @return The dialect for the connection string
         * @throws DocumentException If the dialect cannot be determined
         */
        fun deriveFromConnectionString(connectionString: String): Dialect =
            when {
                connectionString.contains(":sqlite:")     -> SQLITE
                connectionString.contains(":postgresql:") -> POSTGRESQL
                else -> throw DocumentException("Cannot determine dialect from [$connectionString]")
            }
    }
}
