package solutions.bitbadger.documents.common

/**
 * The interface for a document serializer/deserializer
 */
interface DocumentSerializer {

    /**
     * Serialize a document to its JSON representation
     *
     * @param document The document to be serialized
     * @return The JSON representation of the document
     */
    fun <TDoc> serialize(document: TDoc): String

    /**
     * Deserialize a document from its JSON representation
     *
     * @param json The JSON representation of the document
     * @param clazz The class of the document to be deserialized
     * @return The document instance represented by the given JSON string
     */
    fun <TDoc> deserialize(json: String, clazz: Class<TDoc>): TDoc
}
