package solutions.bitbadger.documents.common.query

import solutions.bitbadger.documents.common.Field
import solutions.bitbadger.documents.common.FieldMatch
import solutions.bitbadger.documents.common.query.byFields as byFieldsBase;

/**
 * Functions to count documents
 */
object Count {

    /**
     * Query to count all documents in a table
     *
     * @param tableName The table in which to count documents (may include schema)
     * @return A query to count documents
     */
    fun all(tableName: String) =
        "SELECT COUNT(*) AS it FROM $tableName"

    /**
     * Query to count documents matching the given fields
     *
     * @param tableName The table in which to count documents (may include schema)
     * @param fields The field comparisons for the count
     * @param howMatched How fields should be compared (optional, defaults to ALL)
     * @return A query to count documents matching the given fields
     */
    fun byFields(tableName: String, fields: Collection<Field<*>>, howMatched: FieldMatch? = null) =
        byFieldsBase(all(tableName), fields, howMatched)

    /**
     * Query to count documents via JSON containment (PostgreSQL only)
     *
     * @param tableName The table in which to count documents (may include schema)
     * @return A query to count documents via JSON containment
     */
    fun byContains(tableName: String) =
        statementWhere(all(tableName), Where.jsonContains())

    /**
     * Query to count documents via a JSON path match (PostgreSQL only)
     *
     * @param tableName The table in which to count documents (may include schema)
     * @return A query to count documents via a JSON path match
     */
    fun byJsonPath(tableName: String) =
        statementWhere(all(tableName), Where.jsonPathMatches())
}
