package solutions.bitbadger.documents.common.java;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.common.DocumentException;
import solutions.bitbadger.documents.common.Parameter;
import solutions.bitbadger.documents.common.ParameterType;

import static org.junit.jupiter.api.Assertions.*;

/**
 * Unit tests for the `Parameter` class
 */
@DisplayName("Java | Common | Parameter")
final public class ParameterTest {

    @Test
    @DisplayName("Construction with colon-prefixed name")
    public void ctorWithColon() {
        Parameter<String> p = new Parameter<>(":test", ParameterType.STRING, "ABC");
        assertEquals(":test", p.getName(), "Parameter name was incorrect");
        assertEquals(ParameterType.STRING, p.getType(), "Parameter type was incorrect");
        assertEquals("ABC", p.getValue(), "Parameter value was incorrect");
    }

    @Test
    @DisplayName("Construction with at-sign-prefixed name")
    public void ctorWithAtSign() {
        Parameter<String> p = new Parameter<>("@yo", ParameterType.NUMBER, null);
        assertEquals("@yo", p.getName(), "Parameter name was incorrect");
        assertEquals(ParameterType.NUMBER, p.getType(), "Parameter type was incorrect");
        assertNull(p.getValue(), "Parameter value was incorrect");
    }

    @Test
    @DisplayName("Construction fails with incorrect prefix")
    public void ctorFailsForPrefix() {
        assertThrows(DocumentException.class, () -> new Parameter<>("it", ParameterType.JSON, ""));
    }
}
