package solutions.bitbadger.documents.common

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import kotlin.test.assertEquals
import kotlin.test.assertNotNull
import kotlin.test.assertTrue

/**
 * Unit tests for the `Dialect` enum
 */
@DisplayName("Kotlin | Common | Dialect")
class DialectTest {

    @Test
    @DisplayName("deriveFromConnectionString derives PostgreSQL correctly")
    fun derivesPostgres() =
        assertEquals(
            Dialect.POSTGRESQL, Dialect.deriveFromConnectionString("jdbc:postgresql:db"),
            "Dialect should have been PostgreSQL")

    @Test
    @DisplayName("deriveFromConnectionString derives PostgreSQL correctly")
    fun derivesSQLite() =
        assertEquals(
            Dialect.SQLITE, Dialect.deriveFromConnectionString("jdbc:sqlite:memory"),
            "Dialect should have been SQLite")

    @Test
    @DisplayName("deriveFromConnectionString fails when the connection string is unknown")
    fun deriveFailsWhenUnknown() {
        try {
            Dialect.deriveFromConnectionString("SQL Server")
        } catch (ex: DocumentException) {
            assertNotNull(ex.message, "The exception message should not have been null")
            assertTrue(ex.message!!.contains("[SQL Server]"),
                "The connection string should have been in the exception message")
        }
    }
}
