package solutions.bitbadger.documents.common.query

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.common.Configuration
import solutions.bitbadger.documents.common.Dialect
import solutions.bitbadger.documents.common.DocumentException
import solutions.bitbadger.documents.common.Field
import kotlin.test.assertEquals

/**
 * Unit tests for the `Count` object
 */
@DisplayName("Kotlin | Common | Query: Count")
class CountTest {

    /** Test table name */
    private val tbl = "test_table"

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    fun cleanUp() {
        Configuration.dialectValue = null
    }

    @Test
    @DisplayName("all generates correctly")
    fun all() =
        assertEquals("SELECT COUNT(*) AS it FROM $tbl", Count.all(tbl), "Count query not constructed correctly")

    @Test
    @DisplayName("byFields generates correctly (PostgreSQL)")
    fun byFieldsPostgres() {
        Configuration.dialectValue = Dialect.POSTGRESQL
        assertEquals(
            "SELECT COUNT(*) AS it FROM $tbl WHERE data->>'test' = :field0",
            Count.byFields(tbl, listOf(Field.equal("test", "", ":field0"))),
            "Count query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly (PostgreSQL)")
    fun byFieldsSQLite() {
        Configuration.dialectValue = Dialect.SQLITE
        assertEquals(
            "SELECT COUNT(*) AS it FROM $tbl WHERE data->>'test' = :field0",
            Count.byFields(tbl, listOf(Field.equal("test", "", ":field0"))),
            "Count query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains generates correctly (PostgreSQL)")
    fun byContainsPostgres() {
        Configuration.dialectValue = Dialect.POSTGRESQL
        assertEquals(
            "SELECT COUNT(*) AS it FROM $tbl WHERE data @> :criteria", Count.byContains(tbl),
            "Count query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains fails (SQLite)")
    fun byContainsSQLite() {
        Configuration.dialectValue = Dialect.SQLITE
        assertThrows<DocumentException> { Count.byContains(tbl) }
    }

    @Test
    @DisplayName("byJsonPath generates correctly (PostgreSQL)")
    fun byJsonPathPostgres() {
        Configuration.dialectValue = Dialect.POSTGRESQL
        assertEquals(
            "SELECT COUNT(*) AS it FROM $tbl WHERE jsonb_path_exists(data, :path::jsonpath)",
            Count.byJsonPath(tbl), "Count query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byJsonPath fails (SQLite)")
    fun byJsonPathSQLite() {
        Configuration.dialectValue = Dialect.SQLITE
        assertThrows<DocumentException> { Count.byJsonPath(tbl) }
    }
}
