package solutions.bitbadger.documents.common.query

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.common.Configuration
import solutions.bitbadger.documents.common.Dialect
import solutions.bitbadger.documents.common.DocumentException
import solutions.bitbadger.documents.common.Field
import kotlin.test.assertEquals

/**
 * Unit tests for the `Patch` object
 */
@DisplayName("Kotlin | Common | Query: Patch")
class PatchTest {

    /** Test table name */
    private val tbl = "test_table"

    /**
     * Reset the dialect
     */
    @AfterEach
    fun cleanUp() {
        Configuration.dialectValue = null
    }

    @Test
    @DisplayName("byId generates correctly (PostgreSQL)")
    fun byIdPostgres() {
        Configuration.dialectValue = Dialect.POSTGRESQL
        assertEquals(
            "UPDATE $tbl SET data = data || :data WHERE data->>'id' = :id",
            Patch.byId<String>(tbl), "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byId generates correctly (SQLite)")
    fun byIdSQLite() {
        Configuration.dialectValue = Dialect.SQLITE
        assertEquals(
            "UPDATE $tbl SET data = json_patch(data, json(:data)) WHERE data->>'id' = :id",
            Patch.byId<String>(tbl), "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly (PostgreSQL)")
    fun byFieldsPostgres() {
        Configuration.dialectValue = Dialect.POSTGRESQL
        assertEquals(
            "UPDATE $tbl SET data = data || :data WHERE data->>'z' = :y",
            Patch.byFields(tbl, listOf(Field.equal("z", "", ":y"))),
            "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly (SQLite)")
    fun byFieldsSQLite() {
        Configuration.dialectValue = Dialect.SQLITE
        assertEquals(
            "UPDATE $tbl SET data = json_patch(data, json(:data)) WHERE data->>'z' = :y",
            Patch.byFields(tbl, listOf(Field.equal("z", "", ":y"))),
            "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains generates correctly (PostgreSQL)")
    fun byContainsPostgres() {
        Configuration.dialectValue = Dialect.POSTGRESQL
        assertEquals(
            "UPDATE $tbl SET data = data || :data WHERE data @> :criteria", Patch.byContains(tbl),
            "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains fails (SQLite)")
    fun byContainsSQLite() {
        Configuration.dialectValue = Dialect.SQLITE
        assertThrows<DocumentException> { Patch.byContains(tbl) }
    }

    @Test
    @DisplayName("byJsonPath generates correctly (PostgreSQL)")
    fun byJsonPathPostgres() {
        Configuration.dialectValue = Dialect.POSTGRESQL
        assertEquals(
            "UPDATE $tbl SET data = data || :data WHERE jsonb_path_exists(data, :path::jsonpath)",
            Patch.byJsonPath(tbl), "Patch query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byJsonPath fails (SQLite)")
    fun byJsonPathSQLite() {
        Configuration.dialectValue = Dialect.SQLITE
        assertThrows<DocumentException> { Patch.byJsonPath(tbl) }
    }
}
