package solutions.bitbadger.documents.java.integration

import kotlinx.serialization.Serializable
import solutions.bitbadger.documents.java.insert

/** The test table name to use for integration tests */
const val TEST_TABLE = "test_table"

@Serializable
data class NumIdDocument(val key: Int, val text: String)

@Serializable
data class SubDocument(val foo: String, val bar: String)

@Serializable
data class ArrayDocument(val id: String, val values: List<String>) {
    companion object {
        /** A set of documents used for integration tests */
        val testDocuments = listOf(
            ArrayDocument("first",  listOf("a", "b", "c")),
            ArrayDocument("second", listOf("c", "d", "e")),
            ArrayDocument("third",  listOf("x", "y", "z"))
        )
    }
}

@Serializable
data class JsonDocument(val id: String, val value: String = "", val numValue: Int = 0, val sub: SubDocument? = null) {
    companion object {
        /** Documents to use for testing */
        private val testDocuments = listOf(
            JsonDocument("one",   "FIRST!",   0, null),
            JsonDocument("two",   "another", 10, SubDocument("green", "blue")),
            JsonDocument("three", "",         4, null),
            JsonDocument("four",  "purple",  17, SubDocument("green", "red")),
            JsonDocument("five",  "purple",  18, null)
        )

        fun load(db: ThrowawayDatabase, tableName: String = TEST_TABLE) =
            testDocuments.forEach { db.conn.insert(tableName, it) }
    }
}
