package solutions.bitbadger.documents.java.integration.postgresql

import org.junit.jupiter.api.DisplayName
import solutions.bitbadger.documents.java.integration.common.Document
import kotlin.test.Test

/**
 * PostgreSQL integration tests for the `Document` object / `insert`, `save`, `update` connection extension functions
 */
@DisplayName("Java | Kotlin | PostgreSQL: Document")
class DocumentIT {

    @Test
    @DisplayName("insert works with default values")
    fun insertDefault() =
        PgDB().use(Document::insertDefault)

    @Test
    @DisplayName("insert fails with duplicate key")
    fun insertDupe() =
        PgDB().use(Document::insertDupe)

    @Test
    @DisplayName("insert succeeds with numeric auto IDs")
    fun insertNumAutoId() =
        PgDB().use(Document::insertNumAutoId)

    @Test
    @DisplayName("insert succeeds with UUID auto ID")
    fun insertUUIDAutoId() =
        PgDB().use(Document::insertUUIDAutoId)

    @Test
    @DisplayName("insert succeeds with random string auto ID")
    fun insertStringAutoId() =
        PgDB().use(Document::insertStringAutoId)

    @Test
    @DisplayName("save updates an existing document")
    fun saveMatch() =
        PgDB().use(Document::saveMatch)

    @Test
    @DisplayName("save inserts a new document")
    fun saveNoMatch() =
        PgDB().use(Document::saveNoMatch)

    @Test
    @DisplayName("update replaces an existing document")
    fun updateMatch() =
        PgDB().use(Document::updateMatch)

    @Test
    @DisplayName("update succeeds when no document exists")
    fun updateNoMatch() =
        PgDB().use(Document::updateNoMatch)
}
