package solutions.bitbadger.documents.java.integration.postgresql

import org.junit.jupiter.api.DisplayName
import solutions.bitbadger.documents.java.integration.common.RemoveFields
import kotlin.test.Test

/**
 * PostgreSQL integration tests for the `RemoveFields` object / `removeFieldsBy*` connection extension functions
 */
@DisplayName("Java | Kotlin | PostgreSQL: RemoveFields")
class RemoveFieldsIT {

    @Test
    @DisplayName("byId removes fields from an existing document")
    fun byIdMatchFields() =
        PgDB().use(RemoveFields::byIdMatchFields)

    @Test
    @DisplayName("byId succeeds when fields do not exist on an existing document")
    fun byIdMatchNoFields() =
        PgDB().use(RemoveFields::byIdMatchNoFields)

    @Test
    @DisplayName("byId succeeds when no document exists")
    fun byIdNoMatch() =
        PgDB().use(RemoveFields::byIdNoMatch)

    @Test
    @DisplayName("byFields removes fields from matching documents")
    fun byFieldsMatchFields() =
        PgDB().use(RemoveFields::byFieldsMatchFields)

    @Test
    @DisplayName("byFields succeeds when fields do not exist on matching documents")
    fun byFieldsMatchNoFields() =
        PgDB().use(RemoveFields::byFieldsMatchNoFields)

    @Test
    @DisplayName("byFields succeeds when no matching documents exist")
    fun byFieldsNoMatch() =
        PgDB().use(RemoveFields::byFieldsNoMatch)

    @Test
    @DisplayName("byContains removes fields from matching documents")
    fun byContainsMatchFields() =
        PgDB().use(RemoveFields::byContainsMatchFields)

    @Test
    @DisplayName("byContains succeeds when fields do not exist on matching documents")
    fun byContainsMatchNoFields() =
        PgDB().use(RemoveFields::byContainsMatchNoFields)

    @Test
    @DisplayName("byContains succeeds when no matching documents exist")
    fun byContainsNoMatch() =
        PgDB().use(RemoveFields::byContainsNoMatch)

    @Test
    @DisplayName("byJsonPath removes fields from matching documents")
    fun byJsonPathMatchFields() =
        PgDB().use(RemoveFields::byJsonPathMatchFields)

    @Test
    @DisplayName("byJsonPath succeeds when fields do not exist on matching documents")
    fun byJsonPathMatchNoFields() =
        PgDB().use(RemoveFields::byJsonPathMatchNoFields)

    @Test
    @DisplayName("byJsonPath succeeds when no matching documents exist")
    fun byJsonPathNoMatch() =
        PgDB().use(RemoveFields::byJsonPathNoMatch)
}
