package solutions.bitbadger.documents.java.integration.sqlite

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.common.DocumentException
import solutions.bitbadger.documents.java.integration.common.Exists
import kotlin.test.Test

/**
 * SQLite integration tests for the `Exists` object / `existsBy*` connection extension functions
 */
@DisplayName("Java | Kotlin | SQLite: Exists")
class ExistsIT {

    @Test
    @DisplayName("byId returns true when a document matches the ID")
    fun byIdMatch() =
        SQLiteDB().use(Exists::byIdMatch)

    @Test
    @DisplayName("byId returns false when no document matches the ID")
    fun byIdNoMatch() =
        SQLiteDB().use(Exists::byIdNoMatch)

    @Test
    @DisplayName("byFields returns true when documents match")
    fun byFieldsMatch() =
        SQLiteDB().use(Exists::byFieldsMatch)

    @Test
    @DisplayName("byFields returns false when no documents match")
    fun byFieldsNoMatch() =
        SQLiteDB().use(Exists::byFieldsNoMatch)

    @Test
    @DisplayName("byContains fails")
    fun byContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(Exists::byContainsMatch) }
    }

    @Test
    @DisplayName("byJsonPath fails")
    fun byJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(Exists::byJsonPathMatch) }
    }
}
