package solutions.bitbadger.documents.java.integration.sqlite

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.common.DocumentException
import solutions.bitbadger.documents.java.integration.common.Patch
import kotlin.test.Test

/**
 * SQLite integration tests for the `Patch` object / `patchBy*` connection extension functions
 */
@DisplayName("Java | Kotlin | SQLite: Patch")
class PatchIT {

    @Test
    @DisplayName("byId patches an existing document")
    fun byIdMatch() =
        SQLiteDB().use(Patch::byIdMatch)

    @Test
    @DisplayName("byId succeeds for a non-existent document")
    fun byIdNoMatch() =
        SQLiteDB().use(Patch::byIdNoMatch)

    @Test
    @DisplayName("byFields patches matching document")
    fun byFieldsMatch() =
        SQLiteDB().use(Patch::byFieldsMatch)

    @Test
    @DisplayName("byFields succeeds when no documents match")
    fun byFieldsNoMatch() =
        SQLiteDB().use(Patch::byFieldsNoMatch)

    @Test
    @DisplayName("byContains fails")
    fun byContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(Patch::byContainsMatch) }
    }

    @Test
    @DisplayName("byJsonPath fails")
    fun byJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(Patch::byJsonPathMatch) }
    }
}