package solutions.bitbadger.documents.java;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.common.Dialect;
import solutions.bitbadger.documents.common.DocumentException;

import static org.junit.jupiter.api.Assertions.*;

/**
 * Unit tests for the `Configuration` object
 */
@DisplayName("Java | Configuration")
final public class ConfigurationTest {

    @Test
    @DisplayName("Default JSON options are as expected")
    public void defaultJsonOptions() {
        assertTrue(Configuration.json.getConfiguration().getEncodeDefaults(), "Encode Defaults should have been set");
        assertFalse(Configuration.json.getConfiguration().getExplicitNulls(),
                "Explicit Nulls should not have been set");
        assertTrue(Configuration.json.getConfiguration().getCoerceInputValues(),
                "Coerce Input Values should have been set");
    }

    @Test
    @DisplayName("Default ID field is `id`")
    public void defaultIdField() {
        assertEquals("id", Configuration.idField, "Default ID field incorrect");
    }

    @Test
    @DisplayName("Default Auto ID strategy is `DISABLED`")
    public void defaultAutoId() {
        assertEquals(AutoId.DISABLED, Configuration.autoIdStrategy, "Default Auto ID strategy should be `disabled`");
    }

    @Test
    @DisplayName("Default ID string length should be 16")
    public void defaultIdStringLength() {
        assertEquals(16, Configuration.idStringLength, "Default ID string length should be 16");
    }

    @Test
    @DisplayName("Dialect is derived from connection string")
    public void dialectIsDerived() {
        try {
            assertThrows(DocumentException.class, Configuration::dialect);
            Configuration.setConnectionString("jdbc:postgresql:db");
            assertEquals(Dialect.POSTGRESQL, Configuration.dialect());
        } catch (DocumentException ex) {
            fail(ex);
        } finally {
            Configuration.setConnectionString(null);
        }
    }
}
